@@ -25,6 +25,7 @@ COMPILE_MODES = {
     "simple_stmt": "single",
     "compound_stmt": "single",
 }
+ALLOWED_CHARS = st.characters(codec="utf-8", min_codepoint=1)
 
 
 class PythonIndenter(Indenter):
@@ -37,17 +38,6 @@ class PythonIndenter(Indenter):
     tab_len = 4
 
 
-def utf8_encodable(terminal: str) -> bool:
-    try:
-        terminal.encode()
-        return True
-    except UnicodeEncodeError:  # pragma: no cover
-        # Very rarely, a "." in some terminal regex will generate a surrogate
-        # character that cannot be encoded as UTF-8.  We apply this filter to
-        # ensure it doesn't happen at runtime, but don't worry about coverage.
-        return False
-
-
 class GrammarStrategy(LarkStrategy):
     def __init__(self, grammar: Lark, start: str, auto_target: bool):
         explicit_strategies = {
@@ -55,11 +45,7 @@ class GrammarStrategy(LarkStrategy):
             PythonIndenter.DEDENT_type: st.just(""),
             "NAME": st.text().filter(str.isidentifier),
         }
-        super().__init__(grammar, start, explicit_strategies)
-        self.terminal_strategies = {
-            k: v.map(lambda s: s.replace("\0", "")).filter(utf8_encodable)
-            for k, v in self.terminal_strategies.items()  # type: ignore
-        }
+        super().__init__(grammar, start, explicit_strategies, alphabet=ALLOWED_CHARS)
         self.auto_target = auto_target and start != "single_input"
 
     def do_draw(self, data):  # type: ignore
@@ -80,12 +66,12 @@ class GrammarStrategy(LarkStrategy):
         return result
 
     def draw_symbol(self, data, symbol, draw_state):  # type: ignore
-        count = len(draw_state.result)
+        count = len(draw_state)
         super().draw_symbol(data, symbol, draw_state)
         if symbol.name in COMPILE_MODES:
             try:
                 compile(
-                    source="".join(draw_state.result[count:]),
+                    source="".join(draw_state[count:]),
                     filename="<string>",
                     mode=COMPILE_MODES[symbol.name],
                 )
@@ -108,7 +94,7 @@ class GrammarStrategy(LarkStrategy):
                     # Or the new https://bugs.python.org/issue45738 which makes me
                     # wish CPython would start running proptests in CI already.
                     assume(False)
-                source_code = ascii("".join(draw_state.result[count:]))
+                source_code = ascii("".join(draw_state[count:]))
                 raise type(err)(
                     f"compile({source_code}, '<string>', "
                     f"{COMPILE_MODES[symbol.name]!r}) "
